rm(list=objects())
library(magrittr)


#############################################################################
#######\label{fig:kappa_simu}
#############################################################################
pdf("Figures/kappa_simu.pdf", width=16/2.54, height=16/2.54, pointsize=10)
par(mgp=c(2.2,0.45,0), tcl=-0.4, mar=c(3.3,3.6,1.1,1.1))

par(mfrow=c(2,2)) 
for(dw in c("0","05","10","15"))
{
  fi <- paste0("Results/mlbench1/res_mlbench1_dw_mtry3_", dw, ".RDS")
  res <- readRDS(fi)
  msepred <- colMeans(res$msepred_mat)
  msepred_div <- colMeans(res$msepred_div_mat)
  msepred_kap0 <- colMeans(res$msepred_mat_kap0)
  msepred_div_kap0 <- colMeans(res$msepred_div_mat_kap0)
  
  mse_rf0 <- mean(res$mse_rf)
  plot(msepred, type='l', main=paste0("Diversity weight=", as.numeric(dw)/10), col='blue', ylim=range(msepred, mse_rf0, msepred_div), xlab='m', ylab='mse')
  lines(msepred_div, col='red')
  lines(msepred_div_kap0, col='red', lty='dotted')
  lines(msepred_kap0, col='blue', lty='dotted')
  abline(h=mse_rf0, lty='dotted')
  legend("topright", col=c('blue','red','blue','red','black'), c(expression(F[m][","][kappa]),expression(F[kappa]^"*"), expression(F[m][","][0]), expression(F[0]^"*"), "Random Forest"), bty='n', lty=c(1,1,2,2,2))
  
}
dev.off()


#############################################################################
#######\label{fig:kappa_delta_simu}
#############################################################################

########################################################################gdt08_dw09
file <- "Results/mlbench1/Kappa_inf_1/RF_mtry1_extratrees_grad_step_0.08_Nstep200_dw09.RDS"
res <- readRDS(file)
msepred_gdt08_dw09 <- colMeans(res$msepred_mat)
msepred_div_gdt08_dw09 <- colMeans(res$msepred_div_mat)
msepred_kap0_gdt08_dw09 <- colMeans(res$msepred_mat_kap0)
msepred_div_kap0_gdt08_dw09 <- colMeans(res$msepred_div_mat_kap0)


########################################################################gdt15_dw09
file <- "Results/mlbench1/Kappa_inf_1/RF_mtry1_extratrees_grad_step_0.15_Nstep200_dw09.RDS"
res <- readRDS(file)
msepred_gdt15_dw09 <- colMeans(res$msepred_mat)
msepred_div_gdt15_dw09 <- colMeans(res$msepred_div_mat)
msepred_kap0_gdt15_dw09 <- colMeans(res$msepred_mat_kap0)
msepred_div_kap0_gdt15_dw09 <- colMeans(res$msepred_div_mat_kap0)

########################################################################gdt08_dw05
file <- "Results/mlbench1/Kappa_inf_1/RF_mtry1_extratrees_grad_step_0.08_Nstep200_dw05.RDS"
res <- readRDS(file)
msepred_gdt08_dw05 <- colMeans(res$msepred_mat)
msepred_div_gdt08_dw05 <- colMeans(res$msepred_div_mat)
msepred_kap0_gdt08_dw05 <- colMeans(res$msepred_mat_kap0)
msepred_div_kap0_gdt08_dw05 <- colMeans(res$msepred_div_mat_kap0)

########################################################################gdt15_dw05
file <- "Results/mlbench1/Kappa_inf_1/RF_mtry1_extratrees_grad_step_0.15_Nstep200_dw05.RDS"
res <- readRDS(file)
msepred_gdt15_dw05 <- colMeans(res$msepred_mat)
msepred_div_gdt15_dw05 <- colMeans(res$msepred_div_mat)
msepred_kap0_gdt15_dw05 <- colMeans(res$msepred_mat_kap0)
msepred_div_kap0_gdt15_dw05 <- colMeans(res$msepred_div_mat_kap0)

pdf("Figures/kappa_delta_influence_V2.pdf",  width=16/2.54, height=16/2.54, pointsize=10)
par(mfrow=c(1,1))
par(mgp=c(2.2,0.45,0), tcl=-0.4, mar=c(3.3,3.6,1.1,1.1))

par(mfrow=c(1,1))
col <- yarrr::piratepal("basel")
col <- col[5:7]
plot(msepred_div_gdt08_dw09, type='l', col=col[1], ylim=range(msepred_div_gdt08_dw09,msepred_div_gdt15_dw09,msepred_div_gdt15_dw05), xlab='m'
     , ylab='MSE', lwd=2, lty=2)

lines(msepred_div_gdt15_dw09, col=col[1], lwd=2)


lines(msepred_div_gdt08_dw05, col=col[2], lwd=2, lty=2)
lines(msepred_div_gdt15_dw05, col=col[2],lwd=2)

lines(msepred_div_kap0_gdt08_dw05, col=col[3], lty=2, lwd=2)
lines(msepred_div_kap0_gdt15_dw05, col=col[3], lwd=2)


legend("topright", col=rep(col,each=2), c(expression("F*"[m][","][kappa]["=0.9"][","][delta]["=0.08"]), 
                                          expression("F*"[m][","][kappa]["=0.9"][","][delta]["=0.15"]), 
                                          expression("F*"[m][","][kappa]["=0.5"][","][delta]["=0.08"]),
                                          expression("F*"[m][","][kappa]["=0.5"][","][delta]["=0.15"]),
                                          expression("F*"[m][","][0][","][delta]["=0.08"]), 
                                          expression("F*"[m][","][0][","][delta]["=0.15"])), lty=c(2,1,2,1,2,1), bty='n', lwd=2, cex=2)

dev.off()



#############################################################################
#######\label{fig:learner_influence_simu}
#############################################################################

########################################################################stumps
fi <- "Results/mlbench1/Kappa_inf_1/stumps_grad_step_0.15_Nstep200_dw09.RDS"
res <- readRDS(fi)
stumps_msepred <- colMeans(res$msepred_mat)
stumps_msepred_div <- colMeans(res$msepred_div_mat)

########################################################################PRF
file <- "Results/mlbench1/Kappa_inf_1/RF_mtry1_extratrees_grad_step_0.15_Nstep200_dw09.RDS"
res <- readRDS(file)
msepred_gdt15_dw09 <- colMeans(res$msepred_mat)
msepred_div_gdt15_dw09 <- colMeans(res$msepred_div_mat)

########################################################################random forest 
fi <- "Results/mlbench1/Kappa_inf_1/RF_mtry3_grad_step_0.15_Nstep200_dw09.RDS"
res <- readRDS(fi)
RF_msepred <- colMeans(res$msepred_mat)
RF_msepred_div <- colMeans(res$msepred_div_mat)

col <- yarrr::piratepal("basel")
col <- col[5:7]

pdf("Figures/learner_influence_F_and_F_star.pdf", width=16/2.54, height=16/2.54, pointsize=10)
par(mfrow=c(1,1))
par(mgp=c(2.2,0.45,0), tcl=-0.4, mar=c(3.3,3.6,1.1,1.1))

plot(stumps_msepred_div, type='l', col=col[1], ylim=range(stumps_msepred_div,msepred_div_gdt15_dw09,RF_msepred_div), xlab='m'
     , ylab='MSE', lwd=2)
lines(stumps_msepred, col=col[1], lty=2,  lwd=2)

lines(msepred_div_gdt15_dw09, col=col[2],lwd=2)
lines(msepred_gdt15_dw09, col=col[2],lwd=2,lty=2)

lines(RF_msepred_div, col=col[3],lwd=2)
lines(RF_msepred, col=col[3],lwd=2,lty=2)

legend("topright", col=rep(col[1:3],2), c(expression(Stumps^"*"), expression(PRF^"*"), expression(RF^"*"),"Stumps", "PRF", "RF"),
       lty=c(1,1,1,2,2,2), bty='n', lwd=2, cex=2)
dev.off()


#############################################################################
#######\label{fig:res_RF}
#############################################################################

############################################################################kappa=0.5
fi <- "Results/france_RTE/RF_dw05.RDS"
res <- readRDS(fi)
msepred <- colMeans(res$msepred_mat%>%sqrt)
msepred_div <- colMeans(res$msepred_div_mat%>%sqrt)
msepred_kap0 <- colMeans(res$msepred_mat_kap0%>%sqrt)
msepred_div_kap0 <- colMeans(res$msepred_div_mat_kap0%>%sqrt)
forest <- res$mse_rf%>%sqrt%>%mean


pdf("Figures/RTE_dw05.pdf", width=16/2.54, height=16/2.54, pointsize=10)
par(mfrow=c(1,1))
par(mgp=c(2.2,0.45,0), tcl=-0.4, mar=c(3.3,3.6,1.1,1.1))

plot(msepred, type='l', main=paste0(""), col='blue', ylim=range(msepred, forest, msepred_div, msepred_kap0, 2500), xlab='m', ylab='rmse')
lines(msepred_div, col='red')
lines(msepred_div_kap0, col='red', lty='dotted')
lines(msepred_kap0, col='blue', lty='dotted')
abline(h=forest, lty='dotted')
legend("topright", col=c('blue','red','blue','red','black'), c(expression("F"[m][","][kappa]["=0.5"][","][delta]["=0.08"]), 
                                                               expression("F*"[m][","][kappa]["=0.5"][","][delta]["=0.08"]), 
                                                               expression("F"[m][","][kappa]["=0"][","][delta]["=0.08"]),
                                                               expression("F*"[m][","][kappa]["=0"][","][delta]["=0.08"]),
                                                               "RF"), bty='n', lty=c(1,1,2,2,2), cex=1.5)

dev.off()


############################################################################kappa=0.9
fi <- "Results/france_RTE/RF_dw09.RDS"
res <- readRDS(fi)
msepred <- colMeans(res$msepred_mat%>%sqrt)
msepred_div <- colMeans(res$msepred_div_mat%>%sqrt)
msepred_kap0 <- colMeans(res$msepred_mat_kap0%>%sqrt)
msepred_div_kap0 <- colMeans(res$msepred_div_mat_kap0%>%sqrt)

forest <- res$mse_rf%>%sqrt%>%mean

pdf("Figures/RTE_dw09.pdf", width=16/2.54, height=16/2.54, pointsize=10)
par(mfrow=c(1,1))
par(mgp=c(2.2,0.45,0), tcl=-0.4, mar=c(3.3,3.6,1.1,1.1))

plot(msepred, type='l', main=paste0(""), col='blue', ylim=range(msepred, forest, msepred_div, msepred_kap0, 2500), xlab='m', ylab='rmse')
lines(msepred_div, col='red')
lines(msepred_div_kap0, col='red', lty='dotted')
lines(msepred_kap0, col='blue', lty='dotted')
abline(h=forest, lty='dotted')


legend("topright", col=c('blue','red','blue','red','black'), c(expression("F"[m][","][kappa]["=0.9"][","][delta]["=0.08"]), 
                                                               expression("F*"[m][","][kappa]["=0.9"][","][delta]["=0.08"]), 
                                                               expression("F"[m][","][kappa]["=0"][","][delta]["=0.08"]),
                                                               expression("F*"[m][","][kappa]["=0"][","][delta]["=0.08"]),
                                                               "RF"), bty='n', lty=c(1,1,2,2,2), cex=1.5)

dev.off()



#####################################################################################PRF, kappa=0.5
fi <- "Results/france_RTE/PRF_dw05.RDS"
res <- readRDS(fi)
msepred <- colMeans(res$msepred_mat%>%sqrt)
msepred_div <- colMeans(res$msepred_div_mat%>%sqrt)
msepred_kap0 <- colMeans(res$msepred_mat_kap0%>%sqrt)
msepred_div_kap0 <- colMeans(res$msepred_div_mat_kap0%>%sqrt)

forest <- res$mse_rf%>%sqrt%>%mean

pdf("Figures/RTE_PRF_dw05.pdf", width=16/2.54, height=16/2.54, pointsize=10)
par(mfrow=c(1,1))
par(mgp=c(2.2,0.45,0), tcl=-0.4, mar=c(3.3,3.6,1.1,1.1))

plot(msepred, type='l', main=paste0(""), col='blue', ylim=range(msepred, forest, msepred_div, msepred_kap0, 2500), xlab='m', ylab='rmse')
lines(msepred_div, col='red')
lines(msepred_div_kap0, col='red', lty='dotted')
lines(msepred_kap0, col='blue', lty='dotted')
abline(h=forest, lty='dotted')


legend("topright", col=c('blue','red','blue','red','black'), c(expression("F"[m][","][kappa]["=0.5"][","][delta]["=0.08"]), 
                                                               expression("F*"[m][","][kappa]["=0.5"][","][delta]["=0.08"]), 
                                                               expression("F"[m][","][kappa]["=0"][","][delta]["=0.08"]),
                                                               expression("F*"[m][","][kappa]["=0"][","][delta]["=0.08"]),
                                                               "PF"), bty='n', lty=c(1,1,2,2,2), cex=1.5)


dev.off()

#####################################################################################PRF, kappa=0.5
fi <- "Results/france_RTE/PRF_dw09.RDS"
res <- readRDS(fi)
msepred <- colMeans(res$msepred_mat%>%sqrt)
msepred_div <- colMeans(res$msepred_div_mat%>%sqrt)
msepred_kap0 <- colMeans(res$msepred_mat_kap0%>%sqrt)
msepred_div_kap0 <- colMeans(res$msepred_div_mat_kap0%>%sqrt)

forest <- res$mse_rf%>%sqrt%>%mean

pdf("Figures/RTE_PRF_dw09.pdf", width=16/2.54, height=16/2.54, pointsize=10)
par(mfrow=c(1,1))
par(mgp=c(2.2,0.45,0), tcl=-0.4, mar=c(3.3,3.6,1.1,1.1))

plot(msepred, type='l', main=paste0(""), col='blue', ylim=range(msepred, forest, msepred_div, msepred_kap0, 2500), xlab='m', ylab='rmse')
lines(msepred_div, col='red')
lines(msepred_div_kap0, col='red', lty='dotted')
lines(msepred_kap0, col='blue', lty='dotted')
abline(h=forest, lty='dotted')

legend("topright", col=c('blue','red','blue','red','black'), c(expression("F"[m][","][kappa]["=0.9"][","][delta]["=0.08"]), 
                                                               expression("F*"[m][","][kappa]["=0.9"][","][delta]["=0.08"]), 
                                                               expression("F"[m][","][kappa]["=0"][","][delta]["=0.08"]),
                                                               expression("F*"[m][","][kappa]["=0"][","][delta]["=0.08"]),
                                                               "PF"), bty='n', lty=c(1,1,2,2,2), cex=1.5)
dev.off()






####################################################################################################
###########supplementary material: regression trees
####################################################################################################

########################################################################stump
fi <- "Results/mlbench1/Kappa_inf_1/stumps_grad_step_0.15_Nstep200_dw09.RDS"
res <- readRDS(fi)
msepred_stump<- colMeans(res$msepred_mat)
msepred_div_stump <- colMeans(res$msepred_div_mat)
msepred_kap0_stump <- colMeans(res$msepred_mat_kap0)
msepred_div_kap0_stump <- colMeans(res$msepred_div_mat_kap0)

########################################################################rpart  depth 2
fi <- "Results/mlbench1/rpart_depth_2grad_step_0.15Nstep200_dw09.RDS"
res <- readRDS(fi)
msepred_depth_2 <- colMeans(res$msepred_mat)
msepred_div_depth_2 <- colMeans(res$msepred_div_mat)
msepred_kap0_depth_2 <- colMeans(res$msepred_mat_kap0)
msepred_div_kap0_depth_2 <- colMeans(res$msepred_div_mat_kap0)

msepred_kap0_depth_2
########################################################################rpart  depth 3
fi <- "Results/mlbench1/rpart_depth_3grad_step_0.15Nstep200_dw09.RDS"
res <- readRDS(fi)
msepred_depth_3 <- colMeans(res$msepred_mat)
msepred_div_depth_3 <- colMeans(res$msepred_div_mat)
msepred_kap0_depth_3 <- colMeans(res$msepred_mat_kap0)
msepred_div_kap0_depth_3 <- colMeans(res$msepred_div_mat_kap0)

########################################################################rpart  depth 4
fi <- "Results/mlbench1/rpart_depth_4grad_step_0.15Nstep200_dw09.RDS"
res <- readRDS(fi)
msepred_depth_4 <- colMeans(res$msepred_mat)
msepred_div_depth_4 <- colMeans(res$msepred_div_mat)
msepred_kap0_depth_4 <- colMeans(res$msepred_mat_kap0)
msepred_div_kap0_depth_4 <- colMeans(res$msepred_div_mat_kap0)


pdf("Figures/rpart_depth_comparison.pdf",  width=16/2.54, height=16/2.54, pointsize=10)
par(mfrow=c(1,1))
par(mgp=c(2.2,0.45,0), tcl=-0.4, mar=c(3.3,3.6,1.1,1.1))

par(mfrow=c(1,1))
col <- yarrr::piratepal("basel")
col <- col[4:7]



plot(msepred_kap0_stump, type='l', col=col[1], ylim=range(msepred_div_stump,msepred_div_stump), xlab='m', ylab='MSE', lwd=2, lty=2)
lines(msepred_stump, col=col[1], lwd=2)

lines(msepred_kap0_depth_2, col=col[2], lwd=2, lty=2)
lines(msepred_depth_2, col=col[2],lwd=2)

lines(msepred_kap0_depth_3, col=col[3], lwd=2, lty=2)
lines(msepred_depth_3, col=col[3],lwd=2)

lines(msepred_kap0_depth_4, col=col[4], lwd=2, lty=2)
lines(msepred_depth_4, col=col[4],lwd=2)

legend("topright", col=rep(col,each=2), c(expression("F"[m][","][kappa]["=0.9"][","][delta]["=0.15"][","][depth]["1"]),
                                          expression("F"[m][","][kappa]["=0"][","][delta]["=0.15"][","][depth]["1"]),
                                          expression("F"[m][","][kappa]["=0.9"][","][delta]["=0.15"][","][depth]["2"]),
                                          expression("F"[m][","][kappa]["=0"][","][delta]["=0.15"][","][depth]["2"]),
                                          expression("F"[m][","][kappa]["=0.9"][","][delta]["=0.15"][","][depth]["3"]),
                                          expression("F"[m][","][kappa]["=0"][","][delta]["=0.15"][","][depth]["3"]),
                                          expression("F"[m][","][kappa]["=0.9"][","][delta]["=0.15"][","][depth]["4"]),
                                          expression("F"[m][","][kappa]["=0"][","][delta]["=0.15"][","][depth]["4"])),
       lty=c(1,2,1,2,1,2), bty='n', lwd=2, cex=2)


dev.off()


####################################################################################################
###########figure diversity
####################################################################################################
pdf("Figures/diversity_plots_PRF_V2.pdf",  width=16/2.54, height=16/2.54, pointsize=10)
fi <- "Results/mlbench1/diversity_plots_PRF.RDS"
res_05 <- readRDS(fi)
fi <- "Results/mlbench1/diversity_plots_PRF_09.RDS"
res_09 <- readRDS(fi)

fi <- "Results/mlbench1/diversity_plots_RF_05.RDS"
res_rf_05 <- readRDS(fi)
fi <- "Results/mlbench1/diversity_plots_RF_09.RDS"
res_rf_09 <- readRDS(fi)

col <- yarrr::piratepal("basel")
col <- col[4:7]

plot(colMeans(res_05$diversity_kap0), type='l', lwd=2, col=col[1], ylim=range(res_05$diversity%>%colMeans,res_09$diversity%>%colMeans)
     , xlab="m", ylab="diversity")
lines(colMeans(res_05$diversity), lwd=2, col=col[2])
lines(colMeans(res_09$diversity), lwd=2, col=col[3])

#lines(colMeans(res_05$msepred_div_mat/max(res_05$msepred_div_mat)), lty=2, col=col[2])
lines(colMeans(res_09$msepred_div_mat/max(res_09$msepred_div_mat)), lty=2, col=col[3], lwd=2)


lines(colMeans(res_rf_05$diversity_kap0), lwd=2, col=col[1], lty=3)
lines(colMeans(res_rf_05$diversity), lwd=2, col=col[2], lty=3)
lines(colMeans(res_rf_09$diversity), lwd=2, col=col[3], lty=3)

#lines(colMeans(res_rf_05$msepred_div_mat/max(res_rf_05$msepred_div_mat)), lty=3, col=col[2])
#lines(colMeans(res_rf_09$msepred_div_mat/max(res_rf_09$msepred_div_mat)), lty=3, col=col[3])



#legend("topleft",  col=c(col[1:3], "black", "black"), c("boosting", "diversity boosting-0.5", "diversity boosting-0.9", "PRF", "RF"), bty='n', lty=c(1,1,1,1,3))

legend(x=0, y=1.7 ,  title="Boosting", col=c(col[1:3], "black", "black"), 
       c("boosting", "diversity boosting-0.5", "diversity boosting-0.9"), bty='n', lty=c(1,1,1), title.adj=0)

legend(x=0, y=1.4,  title="Base learner", col=c("black", "black"), c("PRF", "RF"), bty='n', lty=c(1,3), title.adj=0)


dev.off()


