############################################################################################################
###################################################### table
############################################################################################################
library(xtable)

####################################################################################################
###########mlbench simulation, label{simu_table}
####################################################################################################
#κ= 0.9 toencourage diversity and δ= 0.15 so that each of the methods converge after 200steps

comput_tab_simu <- function(res, base_learner="rf")
{
  msepred <- colMeans(res$msepred_mat)
  msepred_div<- colMeans(res$msepred_div_mat)
  
  msepred_kap0 <- colMeans(res$msepred_mat_kap0)
  msepred_div_kap0 <- colMeans(res$msepred_div_mat_kap0)
  
  
  learner <- res[[paste0("mse_", base_learner)]]%>%mean
  
  boosting <- msepred_kap0%>%min
  boosting_agg <- msepred_div_kap0%>%min
  boosting_div <- msepred%>%min
  boosting_div_agg <- msepred_div%>%min
  
  sd_forest <- sd(res[[paste0("mse_", base_learner)]])
  sd_boosting <- apply(res$msepred_mat_kap0%>%sqrt, 1, min)%>%sd
  sd_boosting_agg  <- apply(res$msepred_div_mat_kap0%>%sqrt, 1, min)%>%sd
  sd_boosting_div  <- apply(res$msepred_mat%>%sqrt, 1, min)%>%sd
  sd_boosting_div_agg  <- apply(res$msepred_div_mat%>%sqrt, 1, min)%>%sd
  
  results <- c(boosting, boosting_agg, boosting_div, boosting_div_agg, learner)
  names(results) <-  c("boosting", "boosting_agg", "boosting_div", "boosting_div_agg", "Base Learner")
  results <- rbind(results, c(sd_boosting, sd_boosting_agg, sd_boosting_div, sd_boosting_div_agg, sd_forest))
  rownames(results) <- c("mse", "sd")
  
  return(results)
}

fi <- "Results/mlbench1/Kappa_inf_1/stumps_grad_step_0.15_Nstep200_dw09.RDS"
res <- readRDS(fi)
tab_stump <- comput_tab_simu(res, base_learner="rp")
 

file <- "Results/mlbench1/Kappa_inf_1/RF_mtry1_extratrees_grad_step_0.15_Nstep200_dw09.RDS"
res <- readRDS(file)
tab_PRF <- comput_tab_simu(res, base_learner="rf")

 
fi <- "Results/mlbench1/Kappa_inf_1/RF_mtry3_grad_step_0.15_Nstep200_dw09.RDS"
res <- readRDS(fi)
tab_RF <- comput_tab_simu(res, base_learner="rf")



tab_simu_RMSE <- rbind(tab_stump[1,], tab_PRF[1,], tab_RF[1,])
round(tab_simu_RMSE, digits=2)

options(xtable.comment = FALSE)
stat.desc<-xtable(tab_simu_RMSE, caption="", digits=2, label="simu_table")
print(stat.desc, comment = FALSE)



####################################################################################################
###########load forecasting application (Table 2)
####################################################################################################

comput_tab <- function(res, base_learner="rf")
{
  msepred <- colMeans(res$msepred_mat%>%sqrt)
  msepred_div<- colMeans(res$msepred_div_mat%>%sqrt)
  
  msepred_kap0 <- colMeans(res$msepred_mat_kap0%>%sqrt)
  msepred_div_kap0 <- colMeans(res$msepred_div_mat_kap0%>%sqrt)
  
  
  learner <- res[[paste0("mse_", base_learner)]]%>%sqrt%>%mean
  
  boosting <- msepred_kap0%>%min
  boosting_agg <- msepred_div_kap0%>%min
  boosting_div <- msepred%>%min
  boosting_div_agg <- msepred_div%>%min
  
  sd_forest <- sd(res[[paste0("mse_", base_learner)]]%>%sqrt)
  sd_boosting <- apply(res$msepred_mat_kap0%>%sqrt, 1, min)%>%sd
  sd_boosting_agg  <- apply(res$msepred_div_mat_kap0%>%sqrt, 1, min)%>%sd
  sd_boosting_div  <- apply(res$msepred_mat%>%sqrt, 1, min)%>%sd
  sd_boosting_div_agg  <- apply(res$msepred_div_mat%>%sqrt, 1, min)%>%sd
  
  results <- c(boosting, boosting_agg, boosting_div, boosting_div_agg, learner)
  names(results) <-  c("boosting", "boosting_agg", "boosting_div", "boosting_div_agg", "Base Learner")
  results <- rbind(results, c(sd_boosting, sd_boosting_agg, sd_boosting_div, sd_boosting_div_agg, sd_forest))
  rownames(results) <- c("mse", "sd")

  return(results)
}


fi <- "Results/france_RTE/RF_dw05.RDS"
res <- readRDS(fi)
tab_RF_05 <- comput_tab(res)

fi <- "Results/france_RTE/RF_dw09.RDS"
res <- readRDS(fi)
tab_RF_09 <- comput_tab(res)

fi <- "Results/france_RTE/PRF_dw05.RDS"
res <- readRDS(fi)
tab_PRF_05 <- comput_tab(res)

fi <- "Results/france_RTE/PRF_dw09.RDS"
res <- readRDS(fi)
tab_PRF_09 <- comput_tab(res)

tab_load_RMSE <- rbind(tab_RF_05[1,], tab_RF_09[1,], tab_PRF_05[1,], tab_PRF_09[1,])%>%round(, digits=0)
tab_load_RMSE

options(xtable.comment = FALSE)
stat.desc<-xtable(tab_load_RMSE, caption="MSE (best N) with RF as a base learner", digits=0, label="RF_table")
print(stat.desc, comment = FALSE)




####################################################################################################
###########supplementary material: regression trees
####################################################################################################
fi <- "Results/mlbench1/Kappa_inf_1/stumps_grad_step_0.15_Nstep200_dw09.RDS"
res <- readRDS(fi)
tab_stump <- comput_tab_simu(res, base_learner="rp")

fi <- "Results/mlbench1/rpart_depth_2grad_step_0.15Nstep200_dw09.RDS"
res <- readRDS(fi)
tab_depth2 <- comput_tab_simu(res, base_learner="rp")

fi <- "Results/mlbench1/rpart_depth_3grad_step_0.15Nstep200_dw09.RDS"
res <- readRDS(fi)
tab_depth3 <- comput_tab_simu(res, base_learner="rp")

fi <- "Results/mlbench1/rpart_depth_4grad_step_0.15Nstep200_dw09.RDS"
res <- readRDS(fi)
tab_depth4 <- comput_tab_simu(res, base_learner="rp")



tab_simu_RMSE <- rbind(tab_stump[1,], tab_depth2[1,], tab_depth3[1,], tab_depth4[1,])
round(tab_simu_RMSE, digits=2)

tab_simu_sd <- rbind(tab_stump[2,], tab_depth2[2,], tab_depth3[2,], tab_depth4[2,])
round(tab_simu_sd, digits=2)


options(xtable.comment = FALSE)
stat.desc<-xtable(tab_simu_RMSE, caption="", digits=2, label="simu_table_supmat")
print(stat.desc, comment = FALSE)







