rm(list=objects())
library(mgcv)
library(Bodi)
library(magrittr)
library(ranger)
data <- readRDS(file="Data/Data_RTE_janv2012_oct2020_V2.RDS")
data$Confinement <- (data$Date >= as.POSIXct(strptime("2020-03-16 00:00:00", "%Y-%m-%d %H:%M:%S"), tz="UTC"))
data$DateN <- as.numeric(data$Date)

#range(data$Forecast_RTE_dayahead)
#which(is.na(data$))

sel <- which(data$BH==1)
sel2 <- c(sel-48, sel, sel+48)
Inval <- pmax(c(sel2, which(data$Summer_break!=0), which(data$Christmas_break!=0)),1)
length(Inval)/48
data <- data[-Inval,]

train <- which(data$Date < as.POSIXct(strptime("2019-09-01 00:00:00", "%Y-%m-%d %H:%M:%S"), tz="UTC"))
test <- which(data$Date >= as.POSIXct(strptime("2019-09-01 00:00:00", "%Y-%m-%d %H:%M:%S"), tz="UTC"))
# separation of test set in three
test1 <- which(data$Date >= as.POSIXct(strptime("2019-09-01 00:00:00", "%Y-%m-%d %H:%M:%S"), tz="UTC") &
                 data$Date < as.POSIXct(strptime("2020-03-16 00:00:00", "%Y-%m-%d %H:%M:%S"), tz="UTC"))
test2 <- which(data$Date >= as.POSIXct(strptime("2020-03-16 00:00:00", "%Y-%m-%d %H:%M:%S"), tz="UTC") &
                 data$Date < as.POSIXct(strptime("2020-04-16 00:00:00", "%Y-%m-%d %H:%M:%S"), tz="UTC"))
test3 <- which(data$Date >= as.POSIXct(strptime("2020-04-16 00:00:00", "%Y-%m-%d %H:%M:%S"), tz="UTC"))


#############################################################################################################
#############################################################################################################
data0 <- data[train,]
data1 <- data[test1,]

sel0 <- which(data0$tod==40)
data0 <- data0[sel0,]

sel1 <- which(data1$tod==40)
data1 <- data1[sel1,]

#####naif  benchmark
mean((data1$Load-data1$Load.336)^2)%>%sqrt
mean(abs((data1$Load-data1$Load.336)) / data1$Load)


################################################################################
#############RF and diversity  RF
################################################################################
param <- list(num.trees=100,  mtry=3)
cov <-  "DateN + WeekDays + DLS + toy + Temp + Temp_s95 + Temp_s99 + Temp_s99_min + Temp_s99_max + Load.48 + Load.336"
target <- "Load"
equation <- paste0(target, "~", cov)
param$formula <- equation%>%as.formula
param$data <- data0


##forêt de base
rf0 <- do.call(ranger, args=param, quote = FALSE, envir = parent.frame())
rf0.forecast <- predict(rf0, data=data1)$predictions
mse_rf0 <- mean((data1$Load-rf0.forecast)^2)
sqrt(mse_rf0)
mean(abs((data1$Load-rf0.forecast)) / data1$Load)


######boosting
mse_rf <- NULL
msepred_mat <- NULL
msepred_div_mat <- NULL

msepred_mat_kap0 <- NULL
msepred_div_mat_kap0 <- NULL

rf_div_list <- list()
rf_div_kappa0_list <- list()

diversity_weight <- 0.9
theorical_dw <- F
grad_step  <- 0.08

Nsim <- 100
Nstep <- 200

for(i in c(1:Nsim))
{
  #####compute the benchmark, simple forest
  rf0 <- do.call(ranger, args=param, quote = FALSE, envir = parent.frame())
  rf0.forecast <- predict(rf0, data=data1)$predictions
  mse_rf0 <- mean((data1$Load-rf0.forecast)^2)
  
  #####compute a classical boosting of the forest
  rf_div_kap0<- boosting_diversity(target, cov, data0=data0, data1=data1, sample_size=0.50, grad_step=grad_step, 
                                   diversity_weight=0, Nstep=Nstep, model="rf", sampling="random", param=param, theorical_dw=F)
  
  #####compute the error of the boosted models (without averaging, without diversity)
  msepred_kap0 <- apply(rf_div_kap0$forecast_ensemble, 2, function(x){mean((data1$Load-x)^2)})
  embed_ensemble_kap0 <- rf_div_kap0$forecast_ensemble
  
  #####compute the error of the boosted models (with averaging, without diversity)
  for(j in c(2:ncol(rf_div_kap0$forecast_ensemble)))
  {
    embed_ensemble_kap0[,j] <- rowMeans(rf_div_kap0$forecast_ensemble[,1:j])
  }
  msepred_div_kap0 <- apply(embed_ensemble_kap0, 2, function(x){mean((data1$Load-x)^2)})
  
  #####compute a diversity boosting of the forest
  rf_div<- boosting_diversity(target, cov, data0=data0, data1=data1, sample_size=0.50, grad_step=grad_step, diversity_weight=diversity_weight,
                              Nstep=Nstep, model="rf", sampling="random", param=param, theorical_dw=F)
  
  #####compute the error of the diversity boosted models (without averaging, with diversity)
  msepred <- apply(rf_div$forecast_ensemble, 2, function(x){mean((data1$Load-x)^2)})
  embed_ensemble <- rf_div$forecast_ensemble

  #####compute the error of the diversity boosted models (with averaging, with diversity)
  for(j in c(2:ncol(rf_div$forecast_ensemble)))
  {
    embed_ensemble[,j] <- rowMeans(rf_div$forecast_ensemble[,1:j])
  }
  
  msepred_div <- apply(embed_ensemble, 2, function(x){mean((data1$Load-x)^2)})
  
  msepred_mat <- rbind(msepred_mat, msepred)
  msepred_div_mat <- rbind(msepred_div_mat, msepred_div)
  
  msepred_mat_kap0 <- rbind(msepred_mat_kap0, msepred_kap0)
  msepred_div_mat_kap0 <- rbind(msepred_div_mat_kap0, msepred_div_kap0)
  
  rf_div_list[[i]] <- rf_div$forecast_ensemble
  rf_div_kappa0_list[[i]] <- rf_div_kap0$forecast_ensemble
  
  mse_rf <- c(mse_rf, mse_rf0)
  print(i)
  
}


msepred <- colMeans(msepred_mat)
msepred_div <- colMeans(msepred_div_mat)

msepred_kap0 <- colMeans(msepred_mat_kap0)
msepred_div_kap0 <- colMeans(msepred_div_mat_kap0)


res <- list()
res$msepred_mat <- msepred_mat
res$msepred_div_mat <- msepred_div_mat

res$msepred_mat_kap0 <- msepred_mat_kap0
res$msepred_div_mat_kap0 <- msepred_div_mat_kap0

res$mse_rf <- mse_rf
res$param <- param


#saveRDS(res, file="Results/france_RTE/RF_dw09.RDS")








