rm(list=objects())
library(Bodi)
library(rpart)
library(magrittr)
library(mlbench)
library(yarrr)   ###for colors

##########parameters of trees (stumps)
maxdepth <- 4
control <- rpart.control(minsplit = 2, minbucket = 1, cp = 0,
                         maxcompete = 1, maxsurrogate = 0, usesurrogate = 0,
                         xval = 1,
                         surrogatestyle = 0, maxdepth = maxdepth)

param <- list(control=control)

Nsim <- 100                                               ###number of monte carlo simulations
cov <- "x.1+x.2+x.3+x.4+x.5+x.6+x.7+x.8+x.8+x.9+x.10"     ###covariates of the trees
target <- "y"
param$formula <- paste0(target, "~", cov)%>%as.formula
diversity_weight <- 0.9
theorical_dw <- F
grad_step  <- 0.15

n <- 1200                                                 ###number of observations for the simulated data
n0 <- 200                                                 ###number of observations for the estimation set


mse_rp <- NULL
msepred_mat <- NULL
msepred_div_mat <- NULL

msepred_mat_kap0 <- NULL
msepred_div_mat_kap0 <- NULL

Nstep <- 200

for(i in c(1:Nsim))
{

  DataSim <- mlbench.friedman1(n, sd=1)
  DataSim <- data.frame(y=DataSim$y, x=DataSim$x)

  DataSim0 <- DataSim[1:n0,]
  DataSim1 <- DataSim[(n0+1):n,]

  param$data <- DataSim0

  #####benchmark: basic tree without boosting
  rp0 <- do.call(rpart, args=param, quote = FALSE, envir = parent.frame())
  rp0.forecast <- predict(rp0, newdata=DataSim1)
  mse_rp0 <- mean((DataSim1$y-rp0.forecast)^2)

  #####classical boosting
  rp_div_kap0<- boosting_diversity(target, cov, data0=DataSim0, data1=DataSim1, diversity_weight=0, sample_size=0.50, grad_step=grad_step,
                                   Nstep=Nstep, model="rpart", sampling="random", param=param, theorical_dw=F)

  #####compute the error of the boosted models (without averaging, without diversity)
  msepred_kap0 <- apply(rp_div_kap0$forecast_ensemble, 2, function(x){mean((DataSim1$y-x)^2)})

  #####compute the error of the boosted models (with averaging, without diversity)
  embed_ensemble_kap0 <- rp_div_kap0$forecast_ensemble
  for(j in c(2:ncol(rp_div_kap0$forecast_ensemble)))
  {
    embed_ensemble_kap0[,j] <- rowMeans(rp_div_kap0$forecast_ensemble[,1:j])
  }
  msepred_div_kap0 <- apply(embed_ensemble_kap0, 2, function(x){mean((DataSim1$y-x)^2)})

  #####diversity boosting
  rp_div<- boosting_diversity(target, cov, data0=DataSim0, data1=DataSim1, sample_size=0.50, grad_step=grad_step,
                              diversity_weight=diversity_weight, Nstep=Nstep, model="rpart", sampling="random", param=param, theorical_dw=F)

  #####compute the error of the diversity boosted models (without averaging)
  msepred <- apply(rp_div$forecast_ensemble, 2, function(x){mean((DataSim1$y-x)^2)})

  #####compute the error of the diversity boosted models (with averaging)
  embed_ensemble <- rp_div$forecast_ensemble
  for(j in c(2:ncol(rp_div$forecast_ensemble)))
  {
    embed_ensemble[,j] <- rowMeans(rp_div$forecast_ensemble[,1:j])
  }

  msepred_div <- apply(embed_ensemble, 2, function(x){mean((DataSim1$y-x)^2)})

  msepred_mat <- rbind(msepred_mat, msepred)
  msepred_div_mat <- rbind(msepred_div_mat, msepred_div)

  msepred_mat_kap0 <- rbind(msepred_mat_kap0, msepred_kap0)
  msepred_div_mat_kap0 <- rbind(msepred_div_mat_kap0, msepred_div_kap0)

  mse_rp <- c(mse_rp, mse_rp0)
  print(i)

}


res <- list()
res$msepred_mat <- msepred_mat
res$msepred_div_mat <- msepred_div_mat

res$msepred_mat_kap0 <- msepred_mat_kap0
res$msepred_div_mat_kap0 <- msepred_div_mat_kap0
res$mse_rp <- mse_rp

####save the results
file <- paste0("Results/mlbench1/rpart_depth_", maxdepth, "grad_step_", grad_step, "Nstep", Nstep, "_dw0", 10*diversity_weight, ".RDS")
saveRDS(res, file=file)
