rm(list=objects())
library(Bodi)
library(ranger)
library(magrittr)
library(mlbench)
library(yarrr)   ###for colors


#######parameters of the forest
param <- list(num.trees=100,  mtry=3)

Nsim <- 100
cov <- "x.1+x.2+x.3+x.4+x.5+x.6+x.7+x.8+x.8+x.9+x.10"
target <- "y"
param$formula <- paste0(target, "~", cov)%>%as.formula
diversity_weight <- 0.5
theorical_dw <- F
grad_step  <- 0.08


n <- 1200
n0 <- 200
Nstep <- 200

mse_rf <- NULL
msepred_mat <- NULL
msepred_div_mat <- NULL

msepred_mat_kap0 <- NULL
msepred_div_mat_kap0 <- NULL

diversity_kap0 <- NULL
diversity <- NULL

for(i in c(1:Nsim))
{
  
  DataSim <- mlbench.friedman1(n, sd=1)
  DataSim <- data.frame(y=DataSim$y, x=DataSim$x)
  
  DataSim0 <- DataSim[1:n0,]
  DataSim1 <- DataSim[(n0+1):n,]
  
  param$data <- DataSim0
  
  #####benchmark: basic RF without boosting
  rf0 <- do.call(ranger, args=param, quote = FALSE, envir = parent.frame())
  rf0.forecast <- predict(rf0, data=DataSim1)$predictions
  mse_rf0 <- mean((DataSim1$y-rf0.forecast)^2)
  
  #####classical boosting
  rf_div_kap0<- boosting_diversity(target, cov, data0=DataSim0, data1=DataSim1, diversity_weight=0, sample_size=0.50, grad_step=grad_step,
                                   Nstep=Nstep, model="rf", sampling="random", param=param, theorical_dw=F)
  #####compute the error of the boosted models (without averaging, without diversity)
  msepred_kap0 <- apply(rf_div_kap0$forecast_ensemble, 2, function(x){mean((DataSim1$y-x)^2)})
  embed_ensemble_kap0 <- rf_div_kap0$forecast_ensemble
  diversity_kap0 <- rbind(diversity_kap0, rf_div_kap0$diversiy_oob)
  
  #####compute the error of the boosted models (with averaging, without diversity)
  for(j in c(2:ncol(rf_div_kap0$forecast_ensemble)))
  {
    embed_ensemble_kap0[,j] <- rowMeans(rf_div_kap0$forecast_ensemble[,1:j])
  }
  
  msepred_div_kap0 <- apply(embed_ensemble_kap0, 2, function(x){mean((DataSim1$y-x)^2)})
  
  
  #####diversity boosting
  rf_div<- boosting_diversity(target, cov, data0=DataSim0, data1=DataSim1, sample_size=0.50, grad_step=grad_step,
                              diversity_weight=diversity_weight, Nstep=Nstep, model="rf", sampling="random", param=param, theorical_dw=theorical_dw)
  #####compute the error of the boosted models (without averaging, with diversity)
  msepred <- apply(rf_div$forecast_ensemble, 2, function(x){mean((DataSim1$y-x)^2)})
  embed_ensemble <- rf_div$forecast_ensemble
  diversity <- rbind(diversity, rf_div$diversiy_oob)
  
  #####compute the error of the diversity boosted models (with averaging, with diversity)
  for(j in c(2:ncol(rf_div$forecast_ensemble)))
  {
    embed_ensemble[,j] <- rowMeans(rf_div$forecast_ensemble[,1:j])
  }
  
  msepred_div <- apply(embed_ensemble, 2, function(x){mean((DataSim1$y-x)^2)})
  
  msepred_mat <- rbind(msepred_mat, msepred)
  msepred_div_mat <- rbind(msepred_div_mat, msepred_div)
  
  msepred_mat_kap0 <- rbind(msepred_mat_kap0, msepred_kap0)
  msepred_div_mat_kap0 <- rbind(msepred_div_mat_kap0, msepred_div_kap0)
  
  mse_rf <- c(mse_rf, mse_rf0)
  print(i)
  
}


res <- list()
res$msepred_mat <- msepred_mat
res$msepred_div_mat <- msepred_div_mat

res$msepred_mat_kap0 <- msepred_mat_kap0
res$msepred_div_mat_kap0 <- msepred_div_mat_kap0

res$mse_rf <- mse_rf
res$param <- param

res$grad_step <- grad_step
res$diversity_weight <- diversity_weight
res$theorical_dw <- theorical_dw
res$diversity <- diversity
res$diversity_kap0 <- diversity_kap0



saveRDS(res, file="Results/mlbench1/diversity_plots_RF_05.RDS")


